@@ -1,25 +1,26 @@
-from __future__ import absolute_import, division, print_function
-
-from six.moves import map
-
-
 import datetime
-import dateutil
 import tempfile
+from unittest.mock import Mock
 
+import dateutil.tz
+import dateutil.rrule
 import numpy as np
 import pytest
-import pytz
-
-try:
-    # mock in python 3.3+
-    from unittest import mock
-except ImportError:
-    import mock
 
 from matplotlib.testing.decorators import image_comparison
 import matplotlib.pyplot as plt
+from matplotlib.cbook import MatplotlibDeprecationWarning
 import matplotlib.dates as mdates
+import matplotlib.ticker as mticker
+from matplotlib import rc_context
+
+
+def __has_pytz():
+    try:
+        import pytz
+        return True
+    except ImportError:
+        return False
 
 
 def test_date_numpyx():
@@ -155,7 +156,8 @@ def test_too_many_date_ticks():
     with pytest.warns(UserWarning) as rec:
         ax.set_xlim((t0, tf), auto=True)
         assert len(rec) == 1
-        assert 'Attempting to set identical left==right' in str(rec[0].message)
+        assert \
+            'Attempting to set identical left == right' in str(rec[0].message)
     ax.plot([], [])
     ax.xaxis.set_major_locator(mdates.DayLocator())
     with pytest.raises(RuntimeError):
@@ -189,8 +191,8 @@ def test_RRuleLocator():
 
 def test_RRuleLocator_dayrange():
     loc = mdates.DayLocator()
-    x1 = datetime.datetime(year=1, month=1, day=1, tzinfo=pytz.UTC)
-    y1 = datetime.datetime(year=1, month=1, day=16, tzinfo=pytz.UTC)
+    x1 = datetime.datetime(year=1, month=1, day=1, tzinfo=mdates.UTC)
+    y1 = datetime.datetime(year=1, month=1, day=16, tzinfo=mdates.UTC)
     loc.tick_values(x1, y1)
     # On success, no overflow error shall be thrown
 
@@ -221,6 +223,38 @@ def test_DateFormatter():
     fig.autofmt_xdate()
 
 
+def test_locator_set_formatter():
+    """
+    Test if setting the locator only will update the AutoDateFormatter to use
+    the new locator.
+    """
+    plt.rcParams["date.autoformatter.minute"] = "%d %H:%M"
+    t = [datetime.datetime(2018, 9, 30, 8, 0),
+         datetime.datetime(2018, 9, 30, 8, 59),
+         datetime.datetime(2018, 9, 30, 10, 30)]
+    x = [2, 3, 1]
+
+    fig, ax = plt.subplots()
+    ax.plot(t, x)
+    ax.xaxis.set_major_locator(mdates.MinuteLocator((0, 30)))
+    fig.canvas.draw()
+    ticklabels = [tl.get_text() for tl in ax.get_xticklabels()]
+    expected = ['30 08:00', '30 08:30', '30 09:00',
+                '30 09:30', '30 10:00', '30 10:30']
+    assert ticklabels == expected
+
+    ax.xaxis.set_major_locator(mticker.NullLocator())
+    ax.xaxis.set_minor_locator(mdates.MinuteLocator((5, 55)))
+    decoy_loc = mdates.MinuteLocator((12, 27))
+    ax.xaxis.set_minor_formatter(mdates.AutoDateFormatter(decoy_loc))
+
+    ax.xaxis.set_minor_locator(mdates.MinuteLocator((15, 45)))
+    fig.canvas.draw()
+    ticklabels = [tl.get_text() for tl in ax.get_xticklabels(which="minor")]
+    expected = ['30 08:15', '30 08:45', '30 09:15', '30 09:45', '30 10:15']
+    assert ticklabels == expected
+
+
 def test_date_formatter_strftime():
     """
     Tests that DateFormatter matches datetime.strftime,
@@ -249,7 +283,8 @@ def test_date_formatter_strftime():
                 minute=dt.minute,
                 second=dt.second,
                 microsecond=dt.microsecond))
-        assert formatter.strftime(dt) == formatted_date_str
+        with pytest.warns(MatplotlibDeprecationWarning):
+            assert formatter.strftime(dt) == formatted_date_str
 
         try:
             # Test strftime("%x") with the current locale.
@@ -257,8 +292,9 @@ def test_date_formatter_strftime():
             locale_formatter = mdates.DateFormatter("%x")
             locale_d_fmt = locale.nl_langinfo(locale.D_FMT)
             expanded_formatter = mdates.DateFormatter(locale_d_fmt)
-            assert locale_formatter.strftime(dt) == \
-                expanded_formatter.strftime(dt)
+            with pytest.warns(MatplotlibDeprecationWarning):
+                assert locale_formatter.strftime(dt) == \
+                    expanded_formatter.strftime(dt)
         except (ImportError, AttributeError):
             pass
 
@@ -270,7 +306,7 @@ def test_date_formatter_strftime():
 
 def test_date_formatter_callable():
     scale = -11
-    locator = mock.Mock(_get_unit=mock.Mock(return_value=scale))
+    locator = Mock(_get_unit=Mock(return_value=scale))
     callable_formatting_function = (lambda dates, _:
                                     [dt.strftime('%d-%m//%Y') for dt in dates])
 
@@ -329,7 +365,7 @@ def test_empty_date_with_year_formatter():
 
 def test_auto_date_locator():
     def _create_auto_date_locator(date1, date2):
-        locator = mdates.AutoDateLocator()
+        locator = mdates.AutoDateLocator(interval_multiples=False)
         locator.create_dummy_axis()
         locator.set_view_interval(mdates.date2num(date1),
                                   mdates.date2num(date2))
@@ -405,7 +441,6 @@ def test_auto_date_locator_intmult():
                                   mdates.date2num(date2))
         return locator
 
-    d1 = datetime.datetime(1997, 1, 1)
     results = ([datetime.timedelta(weeks=52 * 200),
                 ['1980-01-01 00:00:00+00:00', '2000-01-01 00:00:00+00:00',
                  '2020-01-01 00:00:00+00:00', '2040-01-01 00:00:00+00:00',
@@ -430,10 +465,12 @@ def test_auto_date_locator_intmult():
                  '1997-05-01 00:00:00+00:00', '1997-05-22 00:00:00+00:00']
                 ],
                [datetime.timedelta(days=40),
-                ['1997-01-01 00:00:00+00:00', '1997-01-08 00:00:00+00:00',
-                 '1997-01-15 00:00:00+00:00', '1997-01-22 00:00:00+00:00',
-                 '1997-01-29 00:00:00+00:00', '1997-02-01 00:00:00+00:00',
-                 '1997-02-08 00:00:00+00:00']
+                ['1997-01-01 00:00:00+00:00', '1997-01-05 00:00:00+00:00',
+                 '1997-01-09 00:00:00+00:00', '1997-01-13 00:00:00+00:00',
+                 '1997-01-17 00:00:00+00:00', '1997-01-21 00:00:00+00:00',
+                 '1997-01-25 00:00:00+00:00', '1997-01-29 00:00:00+00:00',
+                 '1997-02-01 00:00:00+00:00', '1997-02-05 00:00:00+00:00',
+                 '1997-02-09 00:00:00+00:00']
                 ],
                [datetime.timedelta(hours=40),
                 ['1997-01-01 00:00:00+00:00', '1997-01-01 04:00:00+00:00',
@@ -464,12 +501,135 @@ def test_auto_date_locator_intmult():
                 ],
                )
 
+    d1 = datetime.datetime(1997, 1, 1)
     for t_delta, expected in results:
         d2 = d1 + t_delta
         locator = _create_auto_date_locator(d1, d2)
         assert list(map(str, mdates.num2date(locator()))) == expected
 
 
+def test_concise_formatter():
+    def _create_auto_date_locator(date1, date2):
+        fig, ax = plt.subplots()
+
+        locator = mdates.AutoDateLocator(interval_multiples=True)
+        formatter = mdates.ConciseDateFormatter(locator)
+        ax.yaxis.set_major_locator(locator)
+        ax.yaxis.set_major_formatter(formatter)
+        ax.set_ylim(date1, date2)
+        fig.canvas.draw()
+        sts = []
+        for st in ax.get_yticklabels():
+            sts += [st.get_text()]
+        return sts
+
+    d1 = datetime.datetime(1997, 1, 1)
+    results = ([datetime.timedelta(weeks=52 * 200),
+                [str(t) for t in range(1980, 2201, 20)]
+                ],
+               [datetime.timedelta(weeks=52),
+                ['1997', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug',
+                'Sep', 'Oct', 'Nov', 'Dec']
+                ],
+               [datetime.timedelta(days=141),
+                ['Jan', '22', 'Feb', '22', 'Mar', '22', 'Apr', '22',
+                 'May', '22']
+                ],
+               [datetime.timedelta(days=40),
+                ['Jan', '05', '09', '13', '17', '21', '25', '29', 'Feb',
+                 '05', '09']
+                ],
+               [datetime.timedelta(hours=40),
+                ['Jan-01', '04:00', '08:00', '12:00', '16:00', '20:00',
+                 'Jan-02', '04:00', '08:00', '12:00', '16:00']
+                ],
+               [datetime.timedelta(minutes=20),
+                ['00:00', '00:05', '00:10', '00:15', '00:20']
+                ],
+               [datetime.timedelta(seconds=40),
+                ['00:00', '05', '10', '15', '20', '25', '30', '35', '40']
+                ],
+               [datetime.timedelta(seconds=2),
+                ['59.5', '00:00', '00.5', '01.0', '01.5', '02.0', '02.5']
+                ],
+               )
+    for t_delta, expected in results:
+        d2 = d1 + t_delta
+        strings = _create_auto_date_locator(d1, d2)
+        assert strings == expected
+
+
+def test_auto_date_locator_intmult_tz():
+    def _create_auto_date_locator(date1, date2, tz):
+        locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
+        locator.create_dummy_axis()
+        locator.set_view_interval(mdates.date2num(date1),
+                                  mdates.date2num(date2))
+        return locator
+
+    results = ([datetime.timedelta(weeks=52*200),
+                ['1980-01-01 00:00:00-08:00', '2000-01-01 00:00:00-08:00',
+                 '2020-01-01 00:00:00-08:00', '2040-01-01 00:00:00-08:00',
+                 '2060-01-01 00:00:00-08:00', '2080-01-01 00:00:00-08:00',
+                 '2100-01-01 00:00:00-08:00', '2120-01-01 00:00:00-08:00',
+                 '2140-01-01 00:00:00-08:00', '2160-01-01 00:00:00-08:00',
+                 '2180-01-01 00:00:00-08:00', '2200-01-01 00:00:00-08:00']
+                ],
+               [datetime.timedelta(weeks=52),
+                ['1997-01-01 00:00:00-08:00', '1997-02-01 00:00:00-08:00',
+                 '1997-03-01 00:00:00-08:00', '1997-04-01 00:00:00-08:00',
+                 '1997-05-01 00:00:00-07:00', '1997-06-01 00:00:00-07:00',
+                 '1997-07-01 00:00:00-07:00', '1997-08-01 00:00:00-07:00',
+                 '1997-09-01 00:00:00-07:00', '1997-10-01 00:00:00-07:00',
+                 '1997-11-01 00:00:00-08:00', '1997-12-01 00:00:00-08:00']
+                ],
+               [datetime.timedelta(days=141),
+                ['1997-01-01 00:00:00-08:00', '1997-01-22 00:00:00-08:00',
+                 '1997-02-01 00:00:00-08:00', '1997-02-22 00:00:00-08:00',
+                 '1997-03-01 00:00:00-08:00', '1997-03-22 00:00:00-08:00',
+                 '1997-04-01 00:00:00-08:00', '1997-04-22 00:00:00-07:00',
+                 '1997-05-01 00:00:00-07:00', '1997-05-22 00:00:00-07:00']
+                ],
+               [datetime.timedelta(days=40),
+                ['1997-01-01 00:00:00-08:00', '1997-01-05 00:00:00-08:00',
+                 '1997-01-09 00:00:00-08:00', '1997-01-13 00:00:00-08:00',
+                 '1997-01-17 00:00:00-08:00', '1997-01-21 00:00:00-08:00',
+                 '1997-01-25 00:00:00-08:00', '1997-01-29 00:00:00-08:00',
+                 '1997-02-01 00:00:00-08:00', '1997-02-05 00:00:00-08:00',
+                 '1997-02-09 00:00:00-08:00']
+                ],
+               [datetime.timedelta(hours=40),
+                ['1997-01-01 00:00:00-08:00', '1997-01-01 04:00:00-08:00',
+                 '1997-01-01 08:00:00-08:00', '1997-01-01 12:00:00-08:00',
+                 '1997-01-01 16:00:00-08:00', '1997-01-01 20:00:00-08:00',
+                 '1997-01-02 00:00:00-08:00', '1997-01-02 04:00:00-08:00',
+                 '1997-01-02 08:00:00-08:00', '1997-01-02 12:00:00-08:00',
+                 '1997-01-02 16:00:00-08:00']
+                ],
+               [datetime.timedelta(minutes=20),
+                ['1997-01-01 00:00:00-08:00', '1997-01-01 00:05:00-08:00',
+                 '1997-01-01 00:10:00-08:00', '1997-01-01 00:15:00-08:00',
+                 '1997-01-01 00:20:00-08:00']
+                ],
+               [datetime.timedelta(seconds=40),
+                ['1997-01-01 00:00:00-08:00', '1997-01-01 00:00:05-08:00',
+                 '1997-01-01 00:00:10-08:00', '1997-01-01 00:00:15-08:00',
+                 '1997-01-01 00:00:20-08:00', '1997-01-01 00:00:25-08:00',
+                 '1997-01-01 00:00:30-08:00', '1997-01-01 00:00:35-08:00',
+                 '1997-01-01 00:00:40-08:00']
+                ]
+               )
+
+    tz = dateutil.tz.gettz('Canada/Pacific')
+    d1 = datetime.datetime(1997, 1, 1, tzinfo=tz)
+    for t_delta, expected in results:
+        with rc_context({'_internal.classic_mode': False}):
+            d2 = d1 + t_delta
+            locator = _create_auto_date_locator(d1, d2, tz)
+            st = list(map(str, mdates.num2date(locator(), tz=tz)))
+            assert st == expected
+
+
 @image_comparison(baseline_images=['date_inverted_limit'],
                   extensions=['png'])
 def test_date_inverted_limit():
@@ -487,8 +647,8 @@ def test_date_inverted_limit():
 
 def _test_date2num_dst(date_range, tz_convert):
     # Timezones
-    BRUSSELS = pytz.timezone('Europe/Brussels')
-    UTC = pytz.UTC
+    BRUSSELS = dateutil.tz.gettz('Europe/Brussels')
+    UTC = mdates.UTC
 
     # Create a list of timezone-aware datetime objects in UTC
     # Interval is 0b0.0000011 days, to prevent float rounding issues
@@ -518,7 +678,7 @@ def test_date2num_dst():
         subtraction.
         """
         def __sub__(self, other):
-            r = super(dt_tzaware, self).__sub__(other)
+            r = super().__sub__(other)
             tzinfo = getattr(r, 'tzinfo', None)
 
             if tzinfo is not None:
@@ -532,10 +692,10 @@ def test_date2num_dst():
             return r
 
         def __add__(self, other):
-            return self.mk_tzaware(super(dt_tzaware, self).__add__(other))
+            return self.mk_tzaware(super().__add__(other))
 
         def astimezone(self, tzinfo):
-            dt = super(dt_tzaware, self).astimezone(tzinfo)
+            dt = super().astimezone(tzinfo)
             return self.mk_tzaware(dt)
 
         @classmethod
@@ -580,10 +740,7 @@ def test_date2num_dst_pandas(pd):
     _test_date2num_dst(pd.date_range, tz_convert)
 
 
-@pytest.mark.parametrize("attach_tz, get_tz", [
-    (lambda dt, zi: zi.localize(dt), lambda n: pytz.timezone(n)),
-    (lambda dt, zi: dt.replace(tzinfo=zi), lambda n: dateutil.tz.gettz(n))])
-def test_rrulewrapper(attach_tz, get_tz):
+def _test_rrulewrapper(attach_tz, get_tz):
     SYD = get_tz('Australia/Sydney')
 
     dtstart = attach_tz(datetime.datetime(2017, 4, 1, 0), SYD)
@@ -598,6 +755,49 @@ def test_rrulewrapper(attach_tz, get_tz):
     assert act == exp
 
 
+def test_rrulewrapper():
+    def attach_tz(dt, zi):
+        return dt.replace(tzinfo=zi)
+
+    _test_rrulewrapper(attach_tz, dateutil.tz.gettz)
+
+
+@pytest.mark.pytz
+@pytest.mark.skipif(not __has_pytz(), reason="Requires pytz")
+def test_rrulewrapper_pytz():
+    # Test to make sure pytz zones are supported in rrules
+    import pytz
+
+    def attach_tz(dt, zi):
+        return zi.localize(dt)
+
+    _test_rrulewrapper(attach_tz, pytz.timezone)
+
+
+@pytest.mark.pytz
+@pytest.mark.skipif(not __has_pytz(), reason="Requires pytz")
+def test_yearlocator_pytz():
+    import pytz
+
+    tz = pytz.timezone('America/New_York')
+    x = [tz.localize(datetime.datetime(2010, 1, 1))
+            + datetime.timedelta(i) for i in range(2000)]
+    locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
+    locator.create_dummy_axis()
+    locator.set_view_interval(mdates.date2num(x[0])-1.0,
+                              mdates.date2num(x[-1])+1.0)
+
+    np.testing.assert_allclose([733408.208333, 733773.208333, 734138.208333,
+                                734503.208333, 734869.208333,
+                                735234.208333, 735599.208333], locator())
+    expected = ['2009-01-01 00:00:00-05:00',
+                '2010-01-01 00:00:00-05:00', '2011-01-01 00:00:00-05:00',
+                '2012-01-01 00:00:00-05:00', '2013-01-01 00:00:00-05:00',
+                '2014-01-01 00:00:00-05:00', '2015-01-01 00:00:00-05:00']
+    st = list(map(str, mdates.num2date(locator(), tz=tz)))
+    assert st == expected
+
+
 def test_DayLocator():
     with pytest.raises(ValueError):
         mdates.DayLocator(interval=-1)
@@ -622,3 +822,9 @@ def test_tz_utc():
 def test_num2timedelta(x, tdelta):
     dt = mdates.num2timedelta(x)
     assert dt == tdelta
+
+
+def test_datetime64_in_list():
+    dt = [np.datetime64('2000-01-01'), np.datetime64('2001-01-01')]
+    dn = mdates.date2num(dt)
+    assert np.array_equal(dn, [730120.,  730486.])
