@@ -1,16 +1,15 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
 import io
+import re
 
 import numpy as np
+import pytest
+
 import matplotlib
-from matplotlib.testing.decorators import image_comparison, knownfailureif, cleanup
+from matplotlib.testing.decorators import check_figures_equal, image_comparison
 import matplotlib.pyplot as plt
 from matplotlib import mathtext
 
+
 math_tests = [
     r'$a+b+\dot s+\dot{s}+\ldots$',
     r'$x \doteq y$',
@@ -56,7 +55,7 @@ math_tests = [
     '$\\alpha \\beta \\gamma \\delta \\epsilon \\zeta \\eta \\theta \\iota \\lambda \\mu \\nu \\xi \\pi \\kappa \\rho \\sigma \\tau \\upsilon \\phi \\chi \\psi$',
 
     # The examples prefixed by 'mmltt' are from the MathML torture test here:
-        # http://www.mozilla.org/projects/mathml/demo/texvsmml.xhtml
+    # https://developer.mozilla.org/en-US/docs/Mozilla/MathML_Project/MathML_Torture_Test
     r'${x}^{2}{y}^{2}$',
     r'${}_{2}F_{3}$',
     r'$\frac{x+{y}^{2}}{k+1}$',
@@ -83,30 +82,31 @@ math_tests = [
     # mathtex doesn't support array
     # 'mmltt23'    : r'$\left(\begin{array}{cc}\hfill \left(\begin{array}{cc}\hfill a\hfill & \hfill b\hfill \\ \hfill c\hfill & \hfill d\hfill \end{array}\right)\hfill & \hfill \left(\begin{array}{cc}\hfill e\hfill & \hfill f\hfill \\ \hfill g\hfill & \hfill h\hfill \end{array}\right)\hfill \\ \hfill 0\hfill & \hfill \left(\begin{array}{cc}\hfill i\hfill & \hfill j\hfill \\ \hfill k\hfill & \hfill l\hfill \end{array}\right)\hfill \end{array}\right)$',
     # mathtex doesn't support array
-    # 'mmltt24'   : u'$det|\\begin{array}{ccccc}\\hfill {c}_{0}\\hfill & \\hfill {c}_{1}\\hfill & \\hfill {c}_{2}\\hfill & \\hfill \\dots \\hfill & \\hfill {c}_{n}\\hfill \\\\ \\hfill {c}_{1}\\hfill & \\hfill {c}_{2}\\hfill & \\hfill {c}_{3}\\hfill & \\hfill \\dots \\hfill & \\hfill {c}_{n+1}\\hfill \\\\ \\hfill {c}_{2}\\hfill & \\hfill {c}_{3}\\hfill & \\hfill {c}_{4}\\hfill & \\hfill \\dots \\hfill & \\hfill {c}_{n+2}\\hfill \\\\ \\hfill \\u22ee\\hfill & \\hfill \\u22ee\\hfill & \\hfill \\u22ee\\hfill & \\hfill \\hfill & \\hfill \\u22ee\\hfill \\\\ \\hfill {c}_{n}\\hfill & \\hfill {c}_{n+1}\\hfill & \\hfill {c}_{n+2}\\hfill & \\hfill \\dots \\hfill & \\hfill {c}_{2n}\\hfill \\end{array}|>0$',
+    # 'mmltt24'   : r'$det|\begin{array}{ccccc}\hfill {c}_{0}\hfill & \hfill {c}_{1}\hfill & \hfill {c}_{2}\hfill & \hfill \dots \hfill & \hfill {c}_{n}\hfill \\ \hfill {c}_{1}\hfill & \hfill {c}_{2}\hfill & \hfill {c}_{3}\hfill & \hfill \dots \hfill & \hfill {c}_{n+1}\hfill \\ \hfill {c}_{2}\hfill & \hfill {c}_{3}\hfill & \hfill {c}_{4}\hfill & \hfill \dots \hfill & \hfill {c}_{n+2}\hfill \\ \hfill \u22ee\hfill & \hfill \u22ee\hfill & \hfill \u22ee\hfill & \hfill \hfill & \hfill \u22ee\hfill \\ \hfill {c}_{n}\hfill & \hfill {c}_{n+1}\hfill & \hfill {c}_{n+2}\hfill & \hfill \dots \hfill & \hfill {c}_{2n}\hfill \end{array}|>0$',
     r'${y}_{{x}_{2}}$',
     r'${x}_{92}^{31415}+\pi $',
     r'${x}_{{y}_{b}^{a}}^{{z}_{c}^{d}}$',
     r'${y}_{3}^{\prime \prime \prime }$',
-    r"$\left( \xi \left( 1 - \xi \right) \right)$", # Bug 2969451
-    r"$\left(2 \, a=b\right)$", # Sage bug #8125
-    r"$? ! &$", # github issue #466
-    r'$\operatorname{cos} x$', # github issue #553
+    r"$\left( \xi \left( 1 - \xi \right) \right)$",  # Bug 2969451
+    r"$\left(2 \, a=b\right)$",  # Sage bug #8125
+    r"$? ! &$",  # github issue #466
+    r'$\operatorname{cos} x$',  # github issue #553
     r'$\sum _{\genfrac{}{}{0}{}{0\leq i\leq m}{0<j<n}}P\left(i,j\right)$',
     r"$\left\Vert a \right\Vert \left\vert b \right\vert \left| a \right| \left\| b\right\| \Vert a \Vert \vert b \vert$",
     r'$\mathring{A}  \stackrel{\circ}{A}  \AA$',
     r'$M \, M \thinspace M \/ M \> M \: M \; M \ M \enspace M \quad M \qquad M \! M$',
     r'$\Cup$ $\Cap$ $\leftharpoonup$ $\barwedge$ $\rightharpoonup$',
     r'$\dotplus$ $\doteq$ $\doteqdot$ $\ddots$',
-    r'$xyz^kx_kx^py^{p-2} d_i^jb_jc_kd x^j_i E^0 E^0_u$', # github issue #4873
+    r'$xyz^kx_kx^py^{p-2} d_i^jb_jc_kd x^j_i E^0 E^0_u$',  # github issue #4873
     r'${xyz}^k{x}_{k}{x}^{p}{y}^{p-2} {d}_{i}^{j}{b}_{j}{c}_{k}{d} {x}^{j}_{i}{E}^{0}{E}^0_u$',
     r'${\int}_x^x x\oint_x^x x\int_{X}^{X}x\int_x x \int^x x \int_{x} x\int^{x}{\int}_{x} x{\int}^{x}_{x}x$',
     r'testing$^{123}$',
     ' '.join('$\\' + p + '$' for p in sorted(mathtext.Parser._snowflake)),
     r'$6-2$; $-2$; $ -2$; ${-2}$; ${  -2}$; $20^{+3}_{-2}$',
-    r'$\overline{\omega}^x \frac{1}{2}_0^x$', # github issue #5444
-    r'$,$ $.$ $1{,}234{, }567{ , }890$ and $1,234,567,890$', # github issue 5799
-    r'$\left(X\right)_{a}^{b}$', # github issue 7615
+    r'$\overline{\omega}^x \frac{1}{2}_0^x$',  # github issue #5444
+    r'$,$ $.$ $1{,}234{, }567{ , }890$ and $1,234,567,890$',  # github issue 5799
+    r'$\left(X\right)_{a}^{b}$',  # github issue 7615
+    r'$\dfrac{\$100.00}{y}$',  # github issue #1888
 ]
 
 digits = "0123456789"
@@ -157,36 +157,43 @@ for fonts, chars in font_test_specs:
     for set in chars:
         font_tests.append(wrapper % set)
 
-def make_set(basename, fontset, tests, extensions=None):
-    def make_test(filename, test):
-        @image_comparison(baseline_images=[filename], extensions=extensions)
-        def single_test():
-            matplotlib.rcParams['mathtext.fontset'] = fontset
-            fig = plt.figure(figsize=(5.25, 0.75))
-            fig.text(0.5, 0.5, test, horizontalalignment='center', verticalalignment='center')
-        func = single_test
-        func.__name__ = str("test_" + filename)
-        return func
 
-    # We inject test functions into the global namespace, rather than
-    # using a generator, so that individual tests can be run more
-    # easily from the commandline and so each test will have its own
-    # result.
-    for i, test in enumerate(tests):
-        filename = '%s_%s_%02d' % (basename, fontset, i)
-        globals()['test_%s' % filename] = make_test(filename, test)
+@pytest.fixture
+def baseline_images(request, fontset, index):
+    return ['%s_%s_%02d' % (request.param, fontset, index)]
 
-make_set('mathtext', 'cm', math_tests)
-make_set('mathtext', 'stix', math_tests)
-make_set('mathtext', 'stixsans', math_tests)
-make_set('mathtext', 'dejavusans', math_tests)
-make_set('mathtext', 'dejavuserif', math_tests)
 
-make_set('mathfont', 'cm', font_tests, ['png'])
-make_set('mathfont', 'stix', font_tests, ['png'])
-make_set('mathfont', 'stixsans', font_tests, ['png'])
-make_set('mathfont', 'dejavusans', font_tests, ['png'])
-make_set('mathfont', 'dejavuserif', font_tests, ['png'])
+# In the following two tests, use recwarn to suppress warnings regarding the
+# deprecation of \stackrel and \mathcircled.
+
+
+@pytest.mark.parametrize('index, test', enumerate(math_tests),
+                         ids=[str(index) for index in range(len(math_tests))])
+@pytest.mark.parametrize('fontset',
+                         ['cm', 'stix', 'stixsans', 'dejavusans',
+                          'dejavuserif'])
+@pytest.mark.parametrize('baseline_images', ['mathtext'], indirect=True)
+@image_comparison(baseline_images=None)
+def test_mathtext_rendering(baseline_images, fontset, index, test, recwarn):
+    matplotlib.rcParams['mathtext.fontset'] = fontset
+    fig = plt.figure(figsize=(5.25, 0.75))
+    fig.text(0.5, 0.5, test,
+             horizontalalignment='center', verticalalignment='center')
+
+
+@pytest.mark.parametrize('index, test', enumerate(font_tests),
+                         ids=[str(index) for index in range(len(font_tests))])
+@pytest.mark.parametrize('fontset',
+                         ['cm', 'stix', 'stixsans', 'dejavusans',
+                          'dejavuserif'])
+@pytest.mark.parametrize('baseline_images', ['mathfont'], indirect=True)
+@image_comparison(baseline_images=None, extensions=['png'])
+def test_mathfont_rendering(baseline_images, fontset, index, test, recwarn):
+    matplotlib.rcParams['mathtext.fontset'] = fontset
+    fig = plt.figure(figsize=(5.25, 0.75))
+    fig.text(0.5, 0.5, test,
+             horizontalalignment='center', verticalalignment='center')
+
 
 def test_fontinfo():
     import matplotlib.font_manager as font_manager
@@ -196,8 +203,10 @@ def test_fontinfo():
     table = font.get_sfnt_table("head")
     assert table['version'] == (1, 0)
 
-def test_mathtext_exceptions():
-    errors = [
+
+@pytest.mark.parametrize(
+    'math, msg',
+    [
         (r'$\hspace{}$', r'Expected \hspace{n}'),
         (r'$\hspace{foo}$', r'Expected \hspace{n}'),
         (r'$\frac$', r'Expected \frac{num}{den}'),
@@ -206,8 +215,10 @@ def test_mathtext_exceptions():
         (r'$\stackrel{}{}$', r'Expected \stackrel{num}{den}'),
         (r'$\binom$', r'Expected \binom{num}{den}'),
         (r'$\binom{}{}$', r'Expected \binom{num}{den}'),
-        (r'$\genfrac$', r'Expected \genfrac{ldelim}{rdelim}{rulesize}{style}{num}{den}'),
-        (r'$\genfrac{}{}{}{}{}{}$', r'Expected \genfrac{ldelim}{rdelim}{rulesize}{style}{num}{den}'),
+        (r'$\genfrac$',
+         r'Expected \genfrac{ldelim}{rdelim}{rulesize}{style}{num}{den}'),
+        (r'$\genfrac{}{}{}{}{}{}$',
+         r'Expected \genfrac{ldelim}{rdelim}{rulesize}{style}{num}{den}'),
         (r'$\sqrt$', r'Expected \sqrt{value}'),
         (r'$\sqrt f$', r'Expected \sqrt{value}'),
         (r'$\overline$', r'Expected \overline{value}'),
@@ -215,21 +226,41 @@ def test_mathtext_exceptions():
         (r'$\leftF$', r'Expected a delimiter'),
         (r'$\rightF$', r'Unknown symbol: \rightF'),
         (r'$\left(\right$', r'Expected a delimiter'),
-        (r'$\left($', r'Expected "\right"')
-        ]
-
+        (r'$\left($', r'Expected "\right"'),
+        (r'$\dfrac$', r'Expected \dfrac{num}{den}'),
+        (r'$\dfrac{}{}$', r'Expected \dfrac{num}{den}'),
+    ],
+    ids=[
+        'hspace without value',
+        'hspace with invalid value',
+        'frac without parameters',
+        'frac with empty parameters',
+        'stackrel without parameters',
+        'stackrel with empty parameters',
+        'binom without parameters',
+        'binom with empty parameters',
+        'genfrac without parameters',
+        'genfrac with empty parameters',
+        'sqrt without parameters',
+        'sqrt with invalid value',
+        'overline without parameters',
+        'overline with empty parameter',
+        'left with invalid delimiter',
+        'right with invalid delimiter',
+        'unclosed parentheses with sizing',
+        'unclosed parentheses without sizing',
+        'dfrac without parameters',
+        'dfrac with empty parameters',
+    ]
+)
+def test_mathtext_exceptions(math, msg):
     parser = mathtext.MathTextParser('agg')
 
-    for math, msg in errors:
-        try:
-            parser.parse(math)
-        except ValueError as e:
-            exc = str(e).split('\n')
-            assert exc[3].startswith(msg)
-        else:
-            assert False, "Expected '%s', but didn't get it" % msg
+    with pytest.raises(ValueError) as excinfo:
+        parser.parse(math)
+    excinfo.match(re.escape(msg))
+
 
-@cleanup
 def test_single_minus_sign():
     plt.figure(figsize=(0.3, 0.3))
     plt.text(0.5, 0.5, '$-$')
@@ -240,12 +271,13 @@ def test_single_minus_sign():
 
     buff = io.BytesIO()
     plt.savefig(buff, format="rgba", dpi=1000)
-    array = np.fromstring(buff.getvalue(), dtype=np.uint8)
+    array = np.frombuffer(buff.getvalue(), dtype=np.uint8)
 
     # If this fails, it would be all white
     assert not np.all(array == 0xff)
 
 
-if __name__ == '__main__':
-    import nose
-    nose.runmodule(argv=['-s', '--with-doctest'], exit=False)
+@check_figures_equal(extensions=["png"])
+def test_spaces(fig_test, fig_ref):
+    fig_test.subplots().set_title(r"$1\,2\>3\ 4$")
+    fig_ref.subplots().set_title(r"$1\/2\:3~4$")
