@@ -1,7 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
-import six
-
 import matplotlib
 from matplotlib.font_manager import FontProperties
 from matplotlib.testing.decorators import image_comparison
