@@ -20,7 +20,7 @@ Z = (np.cos(x*0.2) + np.sin(y*0.3))
 Zpos = np.ma.masked_less(Z, 0)
 Zneg = np.ma.masked_greater(Z, 0)
 
-fig, (ax1, ax2) = plt.subplots(figsize=(8, 3), ncols=2)
+fig, (ax1, ax2, ax3) = plt.subplots(figsize=(13, 3), ncols=3)
 
 # plot just the positive data and save the
 # color "mappable" object returned by ax1.imshow
@@ -35,6 +35,13 @@ fig.colorbar(pos, ax=ax1)
 neg = ax2.imshow(Zneg, cmap='Reds_r', interpolation='none')
 fig.colorbar(neg, ax=ax2)
 
+# Plot both positive and negative values between +/- 1.2
+pos_neg_clipped = ax3.imshow(Z, cmap='RdBu', vmin=-1.2, vmax=1.2,
+                             interpolation='none')
+# Add minorticks on the colorbar to make it easy to read the
+# values off the colorbar.
+cbar = fig.colorbar(pos_neg_clipped, ax=ax3, extend='both')
+cbar.minorticks_on()
 plt.show()
 
 #############################################################################
@@ -48,7 +55,10 @@ plt.show()
 # in this example:
 
 import matplotlib
+import matplotlib.colorbar
 matplotlib.axes.Axes.imshow
 matplotlib.pyplot.imshow
 matplotlib.figure.Figure.colorbar
 matplotlib.pyplot.colorbar
+matplotlib.colorbar.Colorbar.minorticks_on
+matplotlib.colorbar.Colorbar.minorticks_off
