@@ -3,8 +3,7 @@
 Spectrogram Demo
 ================
 
-Demo of a spectrogram plot
-(:meth:`~.axes.Axes.specgram`).
+Demo of a spectrogram plot (`~.axes.Axes.specgram`).
 """
 import matplotlib.pyplot as plt
 import numpy as np
@@ -18,8 +17,7 @@ s1 = np.sin(2 * np.pi * 100 * t)
 s2 = 2 * np.sin(2 * np.pi * 400 * t)
 
 # create a transient "chirp"
-mask = np.where(np.logical_and(t > 10, t < 12), 1.0, 0.0)
-s2 = s2 * mask
+s2[t <= 10] = s2[12 <= t] = 0
 
 # add some noise into the mix
 nse = 0.01 * np.random.random(size=len(t))
